@@ -4,7 +4,7 @@ from typing import Any, List, Tuple, Union
 
 import numpy as np
 import pandas as pd
-from sklearn.base import BaseEstimator
+from sklearn.base import BaseEstimator, TransformerMixin
 from sklearn.preprocessing import MinMaxScaler, OneHotEncoder, QuantileTransformer
 
 from elapid.config import MaxentConfig, RegularizationConfig
@@ -12,374 +12,8 @@ from elapid.types import ArrayLike, validate_boolean, validate_feature_types, va
 from elapid.utils import make_band_labels, repeat_array
 
 
-class LinearTransformer(MinMaxScaler):
-    """Applies linear feature transformations to rescale features from 0-1."""
-
-    clamp: bool = None
-    feature_range: None
-
-    def __init__(
-        self,
-        clamp: bool = MaxentConfig.clamp,
-        feature_range: Tuple[float, float] = (0.0, 1.0),
-    ):
-        self.clamp = clamp
-        self.feature_range = feature_range
-        super().__init__(clip=clamp, feature_range=feature_range)
-
-
-class QuadraticTransformer(BaseEstimator):
-    """Applies quadtratic feature transformations and rescales features from 0-1."""
-
-    clamp: bool = None
-    feature_range: Tuple[float, float] = None
-    estimator: BaseEstimator = None
-
-    def __init__(
-        self,
-        clamp: bool = MaxentConfig.clamp,
-        feature_range: Tuple[float, float] = (0.0, 1.0),
-    ):
-        self.clamp = clamp
-        self.feature_range = feature_range
-        self.estimator = MinMaxScaler(clip=self.clamp, feature_range=self.feature_range)
-
-    def fit(self, x: ArrayLike) -> None:
-        """Compute the minimum and maximum for scaling.
-
-        Args:
-            x: array-like of shape (n_samples, n_features)
-                The data used to compute the per-feature minimum and maximum
-                used for later scaling along the features axis.
-        Returns:
-            None. Updates the transformer with feature fitting parameters.
-        """
-        self.estimator.fit(np.array(x) ** 2)
-
-    def transform(self, x: ArrayLike) -> np.ndarray:
-        """Scale covariates according to the feature range.
-
-        Args:
-            x: array-like of shape (n_samples, n_features)
-                Input data that will be transformed.
-
-        Returns:
-            ndarray with transformed data.
-        """
-        return self.estimator.transform(np.array(x) ** 2)
-
-    def fit_transform(self, x: ArrayLike) -> np.ndarray:
-        """Fits scaler to x and returns transformed features.
-
-        Args:
-            x: array-like of shape (n_samples, n_features)
-                Input data to fit the scaler and to transform.
-
-        Returns:
-            ndarray with transformed data.
-        """
-        self.fit(x)
-        return self.estimator.transform(np.array(x) ** 2)
-
-    def inverse_transform(self, x: ArrayLike) -> np.ndarray:
-        """Revert from transformed features to original covariate values.
-
-        Args:
-            x: array-like of shape (n_xamples, n_features)
-                Transformed feature data to convert to covariate data.
-
-        Returns:
-            ndarray with unscaled covariate values.
-        """
-        return self.estimator.inverse_transform(np.array(x)) ** 0.5
-
-
-class ProductTransformer(BaseEstimator):
-    """Computes the column-wise product of an array of input features, rescaling from 0-1."""
-
-    clamp: bool = None
-    feature_range: Tuple[float, float] = None
-    estimator: BaseEstimator = None
-
-    def __init__(
-        self,
-        clamp: bool = MaxentConfig.clamp,
-        feature_range: Tuple[float, float] = (0.0, 1.0),
-    ):
-        self.clamp = clamp
-        self.feature_range = feature_range
-        self.estimator = MinMaxScaler(clip=self.clamp, feature_range=self.feature_range)
-
-    def fit(self, x: ArrayLike) -> None:
-        """Compute the minimum and maximum for scaling.
-
-        Args:
-            x: array-like of shape (n_samples, n_features)
-                The data used to compute the per-feature minimum and maximum
-                used for later scaling along the features axis.
-        Returns:
-            None. Updates the transformer with feature fitting parameters.
-        """
-        self.estimator.fit(column_product(np.array(x)))
-
-    def transform(self, x: ArrayLike) -> np.ndarray:
-        """Scale covariates according to the feature range.
-
-        Args:
-            x: array-like of shape (n_samples, n_features)
-                Input data that will be transformed.
-
-        Returns:
-            ndarray with transformed data.
-        """
-        return self.estimator.transform(column_product(np.array(x)))
-
-    def fit_transform(self, x: ArrayLike) -> np.ndarray:
-        """Fits scaler to x and returns transformed features.
-
-        Args:
-            x: array-like of shape (n_samples, n_features)
-                Input data to fit the scaler and to transform.
-
-        Returns:
-            ndarray with transformed data.
-        """
-        self.fit(x)
-        return self.transform(x)
-
-
-class ThresholdTransformer(BaseEstimator):
-    """Applies binary thresholds to each covariate based on n evenly-spaced
-    thresholds across it's min/max range."""
-
-    n_thresholds_: int = None
-    mins_: np.ndarray = None
-    maxs_: np.ndarray = None
-    threshold_indices_: np.ndarray = None
-
-    def __init__(self, n_thresholds: int = MaxentConfig.n_threshold_features):
-        self.n_thresholds_ = n_thresholds
-
-    def fit(self, x: ArrayLike) -> None:
-        """Compute the minimum and maximum for scaling.
-
-        Args:
-            x: array-like of shape (n_samples, n_features)
-                The data used to compute the per-feature minimum and maximum
-                used for later scaling along the features axis.
-        Returns:
-            None. Updates the transformer with feature fitting parameters.
-        """
-        x = np.array(x)
-        self.mins_ = x.min(axis=0)
-        self.maxs_ = x.max(axis=0)
-        self.threshold_indices_ = np.linspace(self.mins_, self.maxs_, self.n_thresholds_)
-
-    def transform(self, x: ArrayLike) -> np.ndarray:
-        """Scale covariates according to the feature range.
-
-        Args:
-            x: array-like of shape (n_samples, n_features)
-                Input data that will be transformed.
-
-        Returns:
-            ndarray with transformed data.
-        """
-        x = np.array(x)
-        xarr = repeat_array(x, len(self.threshold_indices_), axis=-1)
-        tarr = repeat_array(self.threshold_indices_.transpose(), len(x), axis=0)
-        thresh = (xarr > tarr).reshape(x.shape[0], -1)
-        return thresh.astype(np.uint8)
-
-    def fit_transform(self, x: ArrayLike) -> np.ndarray:
-        """Fits scaler to x and returns transformed features.
-
-        Args:
-            x: array-like of shape (n_samples, n_features)
-                Input data to fit the scaler and to transform.
-
-        Returns:
-            ndarray with transformed data.
-        """
-        self.fit(x)
-        return self.transform(x)
-
-
-class HingeTransformer(BaseEstimator):
-    """Fits hinge transformations to an array of covariates."""
-
-    n_hinges_: int = None
-    mins_: np.ndarray = None
-    maxs_: np.ndarray = None
-    hinge_indices_: np.ndarray = None
-
-    def __init__(self, n_hinges: int = MaxentConfig.n_hinge_features):
-        self.n_hinges_ = n_hinges
-
-    def fit(self, x: ArrayLike) -> None:
-        """Compute the minimum and maximum for scaling.
-
-        Args:
-            x: array-like of shape (n_samples, n_features)
-                The data used to compute the per-feature minimum and maximum
-                used for later scaling along the features axis.
-        Returns:
-            None. Updates the transformer with feature fitting parameters.
-        """
-        x = np.array(x)
-        self.mins_ = x.min(axis=0)
-        self.maxs_ = x.max(axis=0)
-        self.hinge_indices_ = np.linspace(self.mins_, self.maxs_, self.n_hinges_)
-
-    def transform(self, x: ArrayLike) -> np.ndarray:
-        """Scale covariates according to the feature range.
-
-        Args:
-            x: array-like of shape (n_samples, n_features)
-                Input data that will be transformed.
-
-        Returns:
-            ndarray with transformed data.
-        """
-        x = np.array(x)
-        xarr = repeat_array(x, self.n_hinges_ - 1, axis=-1)
-        lharr = repeat_array(self.hinge_indices_[:-1].transpose(), len(x), axis=0)
-        rharr = repeat_array(self.hinge_indices_[1:].transpose(), len(x), axis=0)
-        lh = left_hinge(xarr, lharr, self.maxs_)
-        rh = right_hinge(xarr, self.mins_, rharr)
-        return np.concatenate((lh, rh), axis=2).reshape(x.shape[0], -1)
-
-    def fit_transform(self, x: ArrayLike) -> np.ndarray:
-        """Fits scaler to x and returns transformed features.
-
-        Args:
-            x: array-like of shape (n_samples, n_features)
-                Input data to fit the scaler and to transform.
-
-        Returns:
-            ndarray with transformed data.
-        """
-        self.fit(x)
-        return self.transform(x)
-
-
-class CategoricalTransformer(BaseEstimator):
-    """Applies one-hot encoding to categorical covariate datasets."""
-
-    estimators_: list = None
-
-    def __init__(self):
-        pass
-
-    def fit(self, x: ArrayLike) -> None:
-        """Compute the minimum and maximum for scaling.
-
-        Args:
-            x: array-like of shape (n_samples, n_features)
-                The data used to compute the per-feature minimum and maximum
-                used for later scaling along the features axis.
-        Returns:
-            None. Updates the transformer with feature fitting parameters.
-        """
-        self.estimators_ = []
-        x = np.array(x)
-        if x.ndim == 1:
-            estimator = OneHotEncoder(dtype=np.uint8, sparse=False)
-            self.estimators_.append(estimator.fit(x.reshape(-1, 1)))
-        else:
-            nrows, ncols = x.shape
-            for col in range(ncols):
-                xsub = x[:, col].reshape(-1, 1)
-                estimator = OneHotEncoder(dtype=np.uint8, sparse=False)
-                self.estimators_.append(estimator.fit(xsub))
-
-    def transform(self, x: ArrayLike) -> np.ndarray:
-        """Scale covariates according to the feature range.
-
-        Args:
-            x: array-like of shape (n_samples, n_features)
-                Input data that will be transformed.
-
-        Returns:
-            ndarray with transformed data.
-        """
-        x = np.array(x)
-        if x.ndim == 1:
-            estimator = self.estimators_[0]
-            return estimator.transform(x.reshape(-1, 1))
-        else:
-            class_data = []
-            nrows, ncols = x.shape
-            for col in range(ncols):
-                xsub = x[:, col].reshape(-1, 1)
-                estimator = self.estimators_[col]
-                class_data.append(estimator.transform(xsub))
-            return np.concatenate(class_data, axis=1)
-
-    def fit_transform(self, x: ArrayLike) -> np.ndarray:
-        """Fits scaler to x and returns transformed features.
-
-        Args:
-            x: array-like of shape (n_samples, n_features)
-                Input data to fit the scaler and to transform.
-
-        Returns:
-            ndarray with transformed data.
-        """
-        self.fit(x)
-        return self.transform(x)
-
-
-class CumulativeTransformer(QuantileTransformer):
-    """Applies a percentile-based transform to estimate cumulative suitability."""
-
-    def __init__(self):
-        super().__init__(n_quantiles=100, output_distribution="uniform")
-
-
-class MaxentFeatureTransformer(BaseEstimator):
-    """Transforms covariate data into maxent-format feature data."""
-
-    feature_types_: list = None
-    clamp_: bool = None
-    n_hinge_features_: int = None
-    n_threshold_features_: int = None
-    categorical_: list = None
-    continuous_: list = None
-    categorical_pd_: list = None
-    continuous_pd_: list = None
-    labels_: list = None
-    estimators_: dict = {
-        "linear": None,
-        "quadratic": None,
-        "product": None,
-        "threshold": None,
-        "hinge": None,
-        "categorical": None,
-    }
-    feature_names_: list = None
-
-    def __init__(
-        self,
-        feature_types: Union[str, list] = MaxentConfig.feature_types,
-        clamp: bool = MaxentConfig.clamp,
-        n_hinge_features: int = MaxentConfig.n_hinge_features,
-        n_threshold_features: int = MaxentConfig.n_threshold_features,
-    ):
-        """Computes features based on the maxent feature types specified (like linear, quadratic, hinge).
-
-        Implemented using sklearn conventions (with `.fit()` and `.transform()` functions.
-
-        Args:
-            feature_types: list of maxent features to generate.
-            clamp: set feature values to global mins/maxs during prediction
-            n_hinge_features: number of hinge knots to generate
-            n_threshold_features: nuber of threshold features to generate
-        """
-        self.feature_types_ = validate_feature_types(feature_types)
-        self.clamp_ = validate_boolean(clamp)
-        self.n_hinge_features_ = validate_numeric_scalar(n_hinge_features)
-        self.n_threshold_features_ = validate_numeric_scalar(n_threshold_features)
+class FeaturesMixin:
+    """Methods for formatting x data and labels"""
 
     def _format_covariate_data(self, x: ArrayLike) -> Tuple[np.array, np.array]:
         """Reads input x data and formats it to consistent array dtypes.
@@ -445,7 +79,390 @@ class MaxentFeatureTransformer(BaseEstimator):
             else:
                 self.categorical_ = None
 
-    def fit(self, x: ArrayLike, categorical: list = None, labels: list = None) -> None:
+
+class LinearTransformer(MinMaxScaler):
+    """Applies linear feature transformations to rescale features from 0-1."""
+
+    clamp: bool = None
+    feature_range: None
+
+    def __init__(
+        self,
+        clamp: bool = MaxentConfig.clamp,
+        feature_range: Tuple[float, float] = (0.0, 1.0),
+    ):
+        self.clamp = clamp
+        self.feature_range = feature_range
+        super().__init__(clip=clamp, feature_range=feature_range)
+
+
+class QuadraticTransformer(BaseEstimator, TransformerMixin):
+    """Applies quadtratic feature transformations and rescales features from 0-1."""
+
+    clamp: bool = None
+    feature_range: Tuple[float, float] = None
+    estimator: BaseEstimator = None
+
+    def __init__(
+        self,
+        clamp: bool = MaxentConfig.clamp,
+        feature_range: Tuple[float, float] = (0.0, 1.0),
+    ):
+        self.clamp = clamp
+        self.feature_range = feature_range
+
+    def fit(self, x: ArrayLike) -> "QuadraticTransformer":
+        """Compute the minimum and maximum for scaling.
+
+        Args:
+            x: array-like of shape (n_samples, n_features)
+                The data used to compute the per-feature minimum and maximum
+                used for later scaling along the features axis.
+
+        Returns:
+            self. Returns the transformer with fitted parameters.
+        """
+        self.estimator = MinMaxScaler(clip=self.clamp, feature_range=self.feature_range)
+        self.estimator.fit(np.array(x) ** 2)
+
+        return self
+
+    def transform(self, x: ArrayLike) -> np.ndarray:
+        """Scale covariates according to the feature range.
+
+        Args:
+            x: array-like of shape (n_samples, n_features)
+                Input data that will be transformed.
+
+        Returns:
+            ndarray with transformed data.
+        """
+        return self.estimator.transform(np.array(x) ** 2)
+
+    def fit_transform(self, x: ArrayLike) -> np.ndarray:
+        """Fits scaler to x and returns transformed features.
+
+        Args:
+            x: array-like of shape (n_samples, n_features)
+                Input data to fit the scaler and to transform.
+
+        Returns:
+            ndarray with transformed data.
+        """
+        self.fit(x)
+        return self.estimator.transform(np.array(x) ** 2)
+
+    def inverse_transform(self, x: ArrayLike) -> np.ndarray:
+        """Revert from transformed features to original covariate values.
+
+        Args:
+            x: array-like of shape (n_xamples, n_features)
+                Transformed feature data to convert to covariate data.
+
+        Returns:
+            ndarray with unscaled covariate values.
+        """
+        return self.estimator.inverse_transform(np.array(x)) ** 0.5
+
+
+class ProductTransformer(BaseEstimator, TransformerMixin):
+    """Computes the column-wise product of an array of input features, rescaling from 0-1."""
+
+    clamp: bool = None
+    feature_range: Tuple[float, float] = None
+    estimator: BaseEstimator = None
+
+    def __init__(
+        self,
+        clamp: bool = MaxentConfig.clamp,
+        feature_range: Tuple[float, float] = (0.0, 1.0),
+    ):
+        self.clamp = clamp
+        self.feature_range = feature_range
+
+    def fit(self, x: ArrayLike) -> "ProductTransformer":
+        """Compute the minimum and maximum for scaling.
+
+        Args:
+            x: array-like of shape (n_samples, n_features)
+                The data used to compute the per-feature minimum and maximum
+                used for later scaling along the features axis.
+
+        Returns:
+            self. Returns the transformer with fitted parameters.
+        """
+        self.estimator = MinMaxScaler(clip=self.clamp, feature_range=self.feature_range)
+        self.estimator.fit(column_product(np.array(x)))
+
+        return self
+
+    def transform(self, x: ArrayLike) -> np.ndarray:
+        """Scale covariates according to the feature range.
+
+        Args:
+            x: array-like of shape (n_samples, n_features)
+                Input data that will be transformed.
+
+        Returns:
+            ndarray with transformed data.
+        """
+        return self.estimator.transform(column_product(np.array(x)))
+
+    def fit_transform(self, x: ArrayLike) -> np.ndarray:
+        """Fits scaler to x and returns transformed features.
+
+        Args:
+            x: array-like of shape (n_samples, n_features)
+                Input data to fit the scaler and to transform.
+
+        Returns:
+            ndarray with transformed data.
+        """
+        self.fit(x)
+        return self.transform(x)
+
+
+class ThresholdTransformer(BaseEstimator, TransformerMixin):
+    """Applies binary thresholds to each covariate based on n evenly-spaced
+    thresholds across it's min/max range."""
+
+    n_thresholds_: int = None
+    mins_: np.ndarray = None
+    maxs_: np.ndarray = None
+    threshold_indices_: np.ndarray = None
+
+    def __init__(self, n_thresholds: int = MaxentConfig.n_threshold_features):
+        self.n_thresholds_ = n_thresholds
+
+    def fit(self, x: ArrayLike) -> "ThresholdTransformer":
+        """Compute the minimum and maximum for scaling.
+
+        Args:
+            x: array-like of shape (n_samples, n_features)
+                The data used to compute the per-feature minimum and maximum
+                used for later scaling along the features axis.
+
+        Returns:
+            self. Returns the transformer with fitted parameters.
+        """
+        x = np.array(x)
+        self.mins_ = x.min(axis=0)
+        self.maxs_ = x.max(axis=0)
+        self.threshold_indices_ = np.linspace(self.mins_, self.maxs_, self.n_thresholds_)
+
+        return self
+
+    def transform(self, x: ArrayLike) -> np.ndarray:
+        """Scale covariates according to the feature range.
+
+        Args:
+            x: array-like of shape (n_samples, n_features)
+                Input data that will be transformed.
+
+        Returns:
+            ndarray with transformed data.
+        """
+        x = np.array(x)
+        xarr = repeat_array(x, len(self.threshold_indices_), axis=-1)
+        tarr = repeat_array(self.threshold_indices_.transpose(), len(x), axis=0)
+        thresh = (xarr > tarr).reshape(x.shape[0], -1)
+        return thresh.astype(np.uint8)
+
+    def fit_transform(self, x: ArrayLike) -> np.ndarray:
+        """Fits scaler to x and returns transformed features.
+
+        Args:
+            x: array-like of shape (n_samples, n_features)
+                Input data to fit the scaler and to transform.
+
+        Returns:
+            ndarray with transformed data.
+        """
+        self.fit(x)
+        return self.transform(x)
+
+
+class HingeTransformer(BaseEstimator, TransformerMixin):
+    """Fits hinge transformations to an array of covariates."""
+
+    n_hinges_: int = None
+    mins_: np.ndarray = None
+    maxs_: np.ndarray = None
+    hinge_indices_: np.ndarray = None
+
+    def __init__(self, n_hinges: int = MaxentConfig.n_hinge_features):
+        self.n_hinges_ = n_hinges
+
+    def fit(self, x: ArrayLike) -> "HingeTransformer":
+        """Compute the minimum and maximum for scaling.
+
+        Args:
+            x: array-like of shape (n_samples, n_features)
+                The data used to compute the per-feature minimum and maximum
+                used for later scaling along the features axis.
+
+        Returns:
+            self. Updatesd transformer with fitted parameters.
+        """
+        x = np.array(x)
+        self.mins_ = x.min(axis=0)
+        self.maxs_ = x.max(axis=0)
+        self.hinge_indices_ = np.linspace(self.mins_, self.maxs_, self.n_hinges_)
+
+        return self
+
+    def transform(self, x: ArrayLike) -> np.ndarray:
+        """Scale covariates according to the feature range.
+
+        Args:
+            x: array-like of shape (n_samples, n_features)
+                Input data that will be transformed.
+
+        Returns:
+            ndarray with transformed data.
+        """
+        x = np.array(x)
+        xarr = repeat_array(x, self.n_hinges_ - 1, axis=-1)
+        lharr = repeat_array(self.hinge_indices_[:-1].transpose(), len(x), axis=0)
+        rharr = repeat_array(self.hinge_indices_[1:].transpose(), len(x), axis=0)
+        lh = left_hinge(xarr, lharr, self.maxs_)
+        rh = right_hinge(xarr, self.mins_, rharr)
+        return np.concatenate((lh, rh), axis=2).reshape(x.shape[0], -1)
+
+    def fit_transform(self, x: ArrayLike) -> np.ndarray:
+        """Fits scaler to x and returns transformed features.
+
+        Args:
+            x: array-like of shape (n_samples, n_features)
+                Input data to fit the scaler and to transform.
+
+        Returns:
+            ndarray with transformed data.
+        """
+        self.fit(x)
+        return self.transform(x)
+
+
+class CategoricalTransformer(BaseEstimator, TransformerMixin):
+    """Applies one-hot encoding to categorical covariate datasets."""
+
+    estimators_: list = None
+
+    def __init__(self):
+        pass
+
+    def fit(self, x: ArrayLike) -> "CategoricalTransformer":
+        """Compute the minimum and maximum for scaling.
+
+        Args:
+            x: array-like of shape (n_samples, n_features)
+                The data used to compute the per-feature minimum and maximum
+                used for later scaling along the features axis.
+
+        Returns:
+            self. Returns the transformer with fitted parameters.
+        """
+        self.estimators_ = []
+        x = np.array(x)
+        if x.ndim == 1:
+            estimator = OneHotEncoder(dtype=np.uint8, sparse=False)
+            self.estimators_.append(estimator.fit(x.reshape(-1, 1)))
+        else:
+            nrows, ncols = x.shape
+            for col in range(ncols):
+                xsub = x[:, col].reshape(-1, 1)
+                estimator = OneHotEncoder(dtype=np.uint8, sparse=False)
+                self.estimators_.append(estimator.fit(xsub))
+
+        return self
+
+    def transform(self, x: ArrayLike) -> np.ndarray:
+        """Scale covariates according to the feature range.
+
+        Args:
+            x: array-like of shape (n_samples, n_features)
+                Input data that will be transformed.
+
+        Returns:
+            ndarray with transformed data.
+        """
+        x = np.array(x)
+        if x.ndim == 1:
+            estimator = self.estimators_[0]
+            return estimator.transform(x.reshape(-1, 1))
+        else:
+            class_data = []
+            nrows, ncols = x.shape
+            for col in range(ncols):
+                xsub = x[:, col].reshape(-1, 1)
+                estimator = self.estimators_[col]
+                class_data.append(estimator.transform(xsub))
+            return np.concatenate(class_data, axis=1)
+
+    def fit_transform(self, x: ArrayLike) -> np.ndarray:
+        """Fits scaler to x and returns transformed features.
+
+        Args:
+            x: array-like of shape (n_samples, n_features)
+                Input data to fit the scaler and to transform.
+
+        Returns:
+            ndarray with transformed data.
+        """
+        self.fit(x)
+        return self.transform(x)
+
+
+class CumulativeTransformer(QuantileTransformer):
+    """Applies a percentile-based transform to estimate cumulative suitability."""
+
+    def __init__(self):
+        super().__init__(n_quantiles=100, output_distribution="uniform")
+
+
+class MaxentFeatureTransformer(BaseEstimator, TransformerMixin, FeaturesMixin):
+    """Transforms covariate data into maxent-format feature data."""
+
+    feature_types: list = None
+    clamp: bool = None
+    n_hinge_features: int = None
+    n_threshold_features: int = None
+    categorical_: list = None
+    continuous_: list = None
+    categorical_pd_: list = None
+    continuous_pd_: list = None
+    labels_: list = None
+    estimators_: dict = {
+        "linear": None,
+        "quadratic": None,
+        "product": None,
+        "threshold": None,
+        "hinge": None,
+        "categorical": None,
+    }
+    feature_names_: list = None
+
+    def __init__(
+        self,
+        feature_types: Union[str, list] = MaxentConfig.feature_types,
+        clamp: bool = MaxentConfig.clamp,
+        n_hinge_features: int = MaxentConfig.n_hinge_features,
+        n_threshold_features: int = MaxentConfig.n_threshold_features,
+    ):
+        """Computes features based on the maxent feature types specified (like linear, quadratic, hinge).
+
+        Args:
+            feature_types: list of maxent features to generate.
+            clamp: set feature values to global mins/maxs during prediction
+            n_hinge_features: number of hinge knots to generate
+            n_threshold_features: nuber of threshold features to generate
+        """
+        self.feature_types = feature_types
+        self.clamp = clamp
+        self.n_hinge_features = n_hinge_features
+        self.n_threshold_features = n_threshold_features
+
+    def fit(self, x: ArrayLike, categorical: list = None, labels: list = None) -> "MaxentFeatureTransformer":
         """Compute the minimum and maximum for scaling.
 
         Args:
@@ -456,39 +473,44 @@ class MaxentFeatureTransformer(BaseEstimator):
             labels: covariate column labels. ignored if x is a pandas DataFrame
 
         Returns:
-            None. Updates the transformer with feature fitting parameters.
+            self. Returns the transformer with fitted parameters.
         """
+        self.feature_types = validate_feature_types(self.feature_types)
+        self.clamp = validate_boolean(self.clamp)
+        self.n_hinge_features = validate_numeric_scalar(self.n_hinge_features)
+        self.n_threshold_features = validate_numeric_scalar(self.n_threshold_features)
+
         self._format_labels_and_dtypes(x, categorical=categorical, labels=labels)
         con, cat = self._format_covariate_data(x)
         nrows, ncols = con.shape
 
         feature_names = []
-        if "linear" in self.feature_types_:
-            estimator = LinearTransformer(clamp=self.clamp_)
+        if "linear" in self.feature_types:
+            estimator = LinearTransformer(clamp=self.clamp)
             estimator.fit(con)
             self.estimators_["linear"] = estimator
             feature_names += ["linear"] * estimator.n_features_in_
 
-        if "quadratic" in self.feature_types_:
-            estimator = QuadraticTransformer(clamp=self.clamp_)
+        if "quadratic" in self.feature_types:
+            estimator = QuadraticTransformer(clamp=self.clamp)
             estimator.fit(con)
             self.estimators_["quadratic"] = estimator
             feature_names += ["quadratic"] * estimator.estimator.n_features_in_
 
-        if "product" in self.feature_types_:
-            estimator = ProductTransformer(clamp=self.clamp_)
+        if "product" in self.feature_types:
+            estimator = ProductTransformer(clamp=self.clamp)
             estimator.fit(con)
             self.estimators_["product"] = estimator
             feature_names += ["product"] * estimator.estimator.n_features_in_
 
-        if "threshold" in self.feature_types_:
-            estimator = ThresholdTransformer(n_thresholds=self.n_threshold_features_)
+        if "threshold" in self.feature_types:
+            estimator = ThresholdTransformer(n_thresholds=self.n_threshold_features)
             estimator.fit(con)
             self.estimators_["threshold"] = estimator
             feature_names += ["threshold"] * (estimator.n_thresholds_ * ncols)
 
-        if "hinge" in self.feature_types_:
-            estimator = HingeTransformer(n_hinges=self.n_hinge_features_)
+        if "hinge" in self.feature_types:
+            estimator = HingeTransformer(n_hinges=self.n_hinge_features)
             estimator.fit(con)
             self.estimators_["hinge"] = estimator
             feature_names += ["hinge"] * ((estimator.n_hinges_ - 1) * 2 * ncols)
@@ -502,6 +524,8 @@ class MaxentFeatureTransformer(BaseEstimator):
 
         self.feature_names_ = feature_names
 
+        return self
+
     def transform(self, x: ArrayLike) -> np.ndarray:
         """Scale covariates according to the feature range.
 
@@ -515,19 +539,19 @@ class MaxentFeatureTransformer(BaseEstimator):
         con, cat = self._format_covariate_data(x)
         features = []
 
-        if "linear" in self.feature_types_:
+        if "linear" in self.feature_types:
             features.append(self.estimators_["linear"].transform(con))
 
-        if "quadratic" in self.feature_types_:
+        if "quadratic" in self.feature_types:
             features.append(self.estimators_["quadratic"].transform(con))
 
-        if "product" in self.feature_types_:
+        if "product" in self.feature_types:
             features.append(self.estimators_["product"].transform(con))
 
-        if "threshold" in self.feature_types_:
+        if "threshold" in self.feature_types:
             features.append(self.estimators_["threshold"].transform(con))
 
-        if "hinge" in self.feature_types_:
+        if "hinge" in self.feature_types:
             features.append(self.estimators_["hinge"].transform(con))
 
         if cat is not None:
